<#
	.SYNOPSIS
		Get-ISqlAdvancedPropertyInfo
	.DESCRIPTION
		Retrieve advanced properties by service of a computer using WMI
	.PARAMETER computer
		Computer name
	.EXAMPLE
		.\Get-ISqlAdvancedPropertyInfo -computer MyServer
	.INPUTS
		
	.OUTPUTS
		List of SQL Server advanced properties and settings by service name
	.NOTES
		Script rewritten for PowerShell 7 which no longer supports WMI. Should be backward compatibile for PS v5.1
		If will need to run "winrm quickconfig" to set up WinRM remoting services for the CIM Cmdlets to work
		If you receive an error saying you have a network connection set to Public, correct that or
		   if the connection is alread Private run "Enable-PSRemoting -SkipNetworkProfileCheck -Force" to override the error
	.LINK
		https://docs.microsoft.com/en-us/powershell/module/cimcmdlets/get-ciminstance
#> 

param (
	[string]$serverInstance = "$(Read-Host 'Server Instance [e.g. server01\sql2012]')"
)

begin {
	[void][reflection.assembly]::LoadWithPartialName("Microsoft.SqlServer.Smo")
}
process {
	try {
		Write-Verbose "List Properties of a SQL Server instance using WMI..."

		$serverName = $serverInstance.Split("\")[0]
		#$instance = $serverInstance.Split("\")[1]

		$server = new-object Microsoft.SqlServer.Management.Smo.Server $serverInstance
		$ver = $server.Information.Version.Major

		# Create a WMI namespace for SQL Server
		$isSQLSupported = $false
		if ($ver -eq 9) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement'
			$isSQLSupported = $true
		} elseif ($ver -eq 10) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement10'
			$isSQLSupported = $true
		} elseif ($ver -eq 11) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement11'
			$isSQLSupported = $true
		} elseif ($ver -eq 12) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement12'
			$isSQLSupported = $true
		} elseif ($ver -eq 13) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement13'
			$isSQLSupported = $true
		} elseif ($ver -eq 14) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement14'
			$isSQLSupported = $true
		} elseif ($ver -eq 15) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement15'
			$isSQLSupported = $true
		}

		# Retrieve SQL Server advanced properties and settings using WMI
		if ($isSQLSupported) {
			#$output = Get-WmiObject -class sqlserviceadvancedproperty -namespace $WMInamespace -computername $serverName | 
			#	Sort-Object ServiceName, PropertyName | Select-Object ServiceName, PropertyName, PropertyNumValue, PropertyStrValue
			$output = Get-CimInstance -class sqlserviceadvancedproperty -namespace $WMInamespace -computername $serverName | 
				Sort-Object ServiceName, PropertyName | Select-Object ServiceName, PropertyName, PropertyNumValue, PropertyStrValue
		} else {
			Throw "SQL Server versions earlier than 2005 are not valid."
		}
		
		Write-Output $output
	}
	catch [Exception] {
		Write-Error $Error[0]
		$err = $_.Exception
		while ( $err.InnerException ) {
			$err = $err.InnerException
			Write-Output $err.Message
		}
	}
}