<#
	.SYNOPSIS
		Get-ISqlPort
	.DESCRIPTION
		Retrieve SQL Server port configured for use using WMI
	.PARAMETER serverInstance
		SQL Server Instance
	.EXAMPLE
		.\Get-ISqlPort -serverInstance Server01\sql2012
	.INPUTS
	.OUTPUTS
		SQL Server Port #
	.NOTES
		Use server name only to target default instance
		Script rewritten for PowerShell 7 which no longer supports WMI. Should be compatible with PS v5.1.
		If will need to run "winrm quickconfig" to set up WinRM remoting services for the CIM Cmdlets to work
		If you receive an error saying you have a network connection set to Public, correct that or
		   if the connection is alread Private run "Enable-PSRemoting -SkipNetworkProfileCheck -Force" to override the error
	.LINK
		https://docs.microsoft.com/en-us/powershell/module/cimcmdlets/get-ciminstance
#> 

param (
	[string]$serverInstance = "$(Read-Host 'Server Instance [e.g. server01\sql2012]')"
)

begin {
	[void][System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.SMO")
}
process {
	try {
		Write-Verbose "Retrieve SQL Server port configured for use using WMI..."

		$serverName = $serverInstance.Split("\")[0]
		$instance = $serverInstance.Split("\")[1]

		if ($null -eq $instance) {
			$instance = "MSSQLSERVER"
		} 

		$smoServer = New-Object Microsoft.SqlServer.Management.Smo.Server $serverInstance
		$ver = $smoServer.Information.VersionMajor 

		# Create a WMI query
		$WQL = "SELECT PropertyName, PropertyStrVal "
		$WQL += "FROM ServerNetworkProtocolProperty "
		$WQL += "WHERE InstanceName = '" + $instance + "' AND "
		$WQL += "IPAddressName = 'IPAll' AND "
		$WQL += "ProtocolName = 'Tcp'"

		Write-Debug $WQL

		# Create a WMI namespace for SQL Server
		$isSQLSupported = $false
		if ($ver -eq 9) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement'
			$isSQLSupported = $true
		} elseif ($ver -eq 10) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement10'
			$isSQLSupported = $true
		} elseif ($ver -eq 11) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement11'
			$isSQLSupported = $true 
		} elseif ($ver -eq 12) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement12'
			$isSQLSupported = $true 
		} elseif ($ver -eq 13) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement13'
			$isSQLSupported = $true 
		} elseif ($ver -eq 14) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement14'
			$isSQLSupported = $true 
		} elseif ($ver -eq 15) {
			$WMInamespace = 'root\Microsoft\SqlServer\ComputerManagement15'
			$isSQLSupported = $true 
		}

		# Use PowerShell Get-WmiObject to run a WMI query 
		if ($isSQLSupported) {
			# PowerShell 7 does not support WMI, switched to using CIM
			#$output = Get-WmiObject -query $WQL -computerName $serverName -namespace $WMInamespace | 
			$output = Get-CimInstance -query $WQL -computerName $serverName -namespace $WMInamespace | 
			Select-Object PropertyName, PropertyStrVal
		} else {
			$output = "SQL Server version is unsupported"
		}

		Write-Output $output
	}
	catch [Exception] {
		Write-Error $Error[0]
		$err = $_.Exception
		while ( $err.InnerException ) {
			$err = $err.InnerException
			Write-Output $err.Message
		}
	}
}